@@ -16,9 +16,10 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.42 2002/01/31 20:30:25 priikone Exp $ */
+/* $Id: idlist.c,v 1.46 2002/03/24 14:18:42 priikone Exp $ */
 
-#include "clientlibincludes.h"
+#include "silcincludes.h"
+#include "silcclient.h"
 #include "client_internal.h"
 
 /******************************************************************************
@@ -40,7 +41,7 @@ SilcClientEntry *silc_client_get_clients_local(SilcClient client,
 					       SilcClientConnection conn,
 					       const char *nickname,
 					       const char *format,
-					       uint32 *clients_count)
+					       SilcUInt32 *clients_count)
 {
   SilcIDCacheEntry id_cache;
   SilcIDCacheList list = NULL;
@@ -122,7 +123,7 @@ SILC_CLIENT_CMD_FUNC(get_client_callback)
 {
   GetClientInternal i = (GetClientInternal)context;
   SilcClientEntry *clients;
-  uint32 clients_count;
+  SilcUInt32 clients_count;
 
   /* Get the clients */
   clients = silc_client_get_clients_local(i->client, i->conn,
@@ -282,7 +283,7 @@ SilcClientEntry silc_idlist_get_client(SilcClient client,
 typedef struct {
   SilcClient client;
   SilcClientConnection conn;
-  uint32 list_count;
+  SilcUInt32 list_count;
   SilcBuffer client_id_list;
   SilcGetClientCallback completion;
   void *context;
@@ -294,20 +295,20 @@ SILC_CLIENT_CMD_FUNC(get_clients_list_callback)
   SilcIDCacheEntry id_cache = NULL;
   SilcBuffer client_id_list = i->client_id_list;
   SilcClientEntry *clients = NULL;
-  uint32 clients_count = 0;
+  SilcUInt32 clients_count = 0;
   bool found = FALSE;
   int c;
 
   SILC_LOG_DEBUG(("Start"));
 
   for (c = 0; c < i->list_count; c++) {
-    uint16 idp_len;
+    SilcUInt16 idp_len;
     SilcClientID *client_id;
 
     /* Get Client ID */
     SILC_GET16_MSB(idp_len, client_id_list->data + 2);
     idp_len += 4;
-    client_id = silc_id_payload_parse_id(client_id_list->data, idp_len);
+    client_id = silc_id_payload_parse_id(client_id_list->data, idp_len, NULL);
     if (!client_id) {
       silc_buffer_pull(client_id_list, idp_len);
       continue;
@@ -351,7 +352,7 @@ SILC_CLIENT_CMD_FUNC(get_clients_list_callback)
 
 void silc_client_get_clients_by_list(SilcClient client,
 				     SilcClientConnection conn,
-				     uint32 list_count,
+				     SilcUInt32 list_count,
 				     SilcBuffer client_id_list,
 				     SilcGetClientCallback completion,
 				     void *context)
@@ -359,7 +360,7 @@ void silc_client_get_clients_by_list(SilcClient client,
   SilcIDCacheEntry id_cache = NULL;
   int i;
   unsigned char **res_argv = NULL;
-  uint32 *res_argv_lens = NULL, *res_argv_types = NULL, res_argc = 0;
+  SilcUInt32 *res_argv_lens = NULL, *res_argv_types = NULL, res_argc = 0;
   GetClientsByListInternal in;
 
   SILC_LOG_DEBUG(("Start"));
@@ -373,14 +374,14 @@ void silc_client_get_clients_by_list(SilcClient client,
   in->context = context;
 
   for (i = 0; i < list_count; i++) {
-    uint16 idp_len;
+    SilcUInt16 idp_len;
     SilcClientID *client_id;
     SilcClientEntry entry;
 
     /* Get Client ID */
     SILC_GET16_MSB(idp_len, client_id_list->data + 2);
     idp_len += 4;
-    client_id = silc_id_payload_parse_id(client_id_list->data, idp_len);
+    client_id = silc_id_payload_parse_id(client_id_list->data, idp_len, NULL);
     if (!client_id) {
       silc_buffer_pull(client_id_list, idp_len);
       continue;
@@ -566,7 +567,7 @@ void silc_client_get_client_by_id_resolve(SilcClient client,
 SilcClientEntry
 silc_client_add_client(SilcClient client, SilcClientConnection conn,
 		       char *nickname, char *username, 
-		       char *userinfo, SilcClientID *id, uint32 mode)
+		       char *userinfo, SilcClientID *id, SilcUInt32 mode)
 {
   SilcClientEntry client_entry;
   char *nick = NULL;
@@ -614,7 +615,7 @@ void silc_client_update_client(SilcClient client,
 			       const char *nickname,
 			       const char *username,
 			       const char *userinfo,
-			       uint32 mode)
+			       SilcUInt32 mode)
 {
   char *nick = NULL;
 
@@ -688,7 +689,7 @@ bool silc_client_del_client(SilcClient client, SilcClientConnection conn,
 SilcChannelEntry silc_client_add_channel(SilcClient client,
 					 SilcClientConnection conn,
 					 const char *channel_name,
-					 uint32 mode, 
+					 SilcUInt32 mode, 
 					 SilcChannelID *channel_id)
 {
   SilcChannelEntry channel;
@@ -722,6 +723,8 @@ static void silc_client_del_channel_foreach(void *key, void *context,
 {
   SilcChannelUser chu = (SilcChannelUser)context;
 
+  SILC_LOG_DEBUG(("Start"));
+
   /* Remove the context from the client's channel hash table as that
      table and channel's user_list hash table share this same context. */
   silc_hash_table_del(chu->client->channels, chu->channel);
@@ -735,6 +738,8 @@ bool silc_client_del_channel(SilcClient client, SilcClientConnection conn,
 {
   bool ret = silc_idcache_del_by_context(conn->channel_cache, channel);
 
+  SILC_LOG_DEBUG(("Start"));
+
   /* Free all client entrys from the users list. The silc_hash_table_free
      will free all the entries so they are not freed at the foreach 
      callback. */
@@ -936,6 +941,39 @@ SilcServerEntry silc_client_get_server_by_id(SilcClient client,
   return entry;
 }
 
+/* Add new server entry */
+
+SilcServerEntry silc_client_add_server(SilcClient client,
+				       SilcClientConnection conn,
+				       const char *server_name,
+				       const char *server_info,
+				       SilcServerID *server_id)
+{
+  SilcServerEntry server_entry;
+
+  server_entry = silc_calloc(1, sizeof(*server_entry));
+  if (!server_entry || !server_id)
+    return NULL;
+
+  server_entry->server_id = server_id;
+  if (server_name)
+    server_entry->server_name = strdup(server_name);
+  if (server_info)
+    server_entry->server_info = strdup(server_info);
+
+  /* Add server to cache */
+  if (!silc_idcache_add(conn->server_cache, server_entry->server_name,
+			server_entry->server_id, server_entry, 0, NULL)) {
+    silc_free(server_entry->server_id);
+    silc_free(server_entry->server_name);
+    silc_free(server_entry->server_info);
+    silc_free(server_entry);
+    return NULL;
+  }
+
+  return server_entry;
+}
+
 /* Removes server from the cache by the server entry. */
 
 bool silc_client_del_server(SilcClient client, SilcClientConnection conn,
@@ -962,7 +1000,7 @@ void silc_client_nickname_format(SilcClient client,
   char *newnick = NULL;
   int i, off = 0, len;
   SilcClientEntry *clients;
-  uint32 clients_count = 0;
+  SilcUInt32 clients_count = 0;
   SilcClientEntry unformatted = NULL;
 
   SILC_LOG_DEBUG(("Start"));
